@@ -37,13 +37,14 @@ KNetAttach::KNetAttach( QWidget* parent )
     setupUi( this );
 
     connect(_recent, SIGNAL(toggled(bool)), _recentConnectionName, SLOT(setEnabled(bool)));
+    connect(_connectionName, SIGNAL(textChanged(const QString&)), this, SLOT(updateParametersPageStatus()));
     connect(_user, SIGNAL(textChanged(const QString&)), this, SLOT(updateParametersPageStatus()));
     connect(_host, SIGNAL(textChanged(const QString&)), this, SLOT(updateParametersPageStatus()));
     connect(_path, SIGNAL(textChanged(const QString&)), this, SLOT(updateParametersPageStatus()));
     connect(_useEncryption, SIGNAL(toggled(bool)), this, SLOT(updatePort(bool)));
     connect(_createIcon, SIGNAL(toggled(bool)), this, SLOT(updateFinishButtonText(bool)));
     connect( this, SIGNAL(helpClicked() ), this, SLOT( slotHelpClicked() ) );
-    setIcon(SmallIcon("knetattach"));
+    setWindowIcon(KIcon("knetattach"));
     disconnect(finishButton(), SIGNAL(clicked()), (QDialog*)this, SLOT(accept()));
     connect(finishButton(), SIGNAL(clicked()), this, SLOT(finished()));
     finishButton()->setText(i18n("Save && C&onnect"));
@@ -186,9 +187,12 @@ void KNetAttach::finished()
     url.setHost(_host->text().trimmed());
     url.setUser(_user->text().trimmed());
     QString path = _path->text().trimmed();
+#ifndef Q_WS_WIN
+    // could a relative path really be made absolute by simply prepending a '/' ?
     if (!path.startsWith('/')) {
 	path = QString("/") + path;
     }
+#endif
     url.setPath(path);
    _folderParameters->setEnabled(false);
     bool success = doConnectionTest(url);
