@@ -44,7 +44,9 @@ class NotifyBySound : public KNotifyPlugin
 	public:
 		void setVolume( int v );
 
-		
+	protected:
+		void timerEvent(QTimerEvent *);
+
 	private Q_SLOTS:
 		void slotSoundFinished(int id);
 };
