@@ -1,8 +1,5 @@
 /*
-   $Id: sourceheader 511311 2006-02-19 14:51:05Z trueg $
-
-   This file is part of the Strigi project.
-   Copyright (C) 2007 Sebastian Trueg <trueg@kde.org>
+   Copyright (C) 2007-2008 Sebastian Trueg <trueg@kde.org>
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License as
@@ -26,6 +23,9 @@
 #include <Soprano/Soprano>
 #include <Soprano/Index/IndexFilterModel>
 #include <Soprano/Index/CLuceneIndex>
+#include <Soprano/Vocabulary/RDF>
+#include <Soprano/Vocabulary/Xesam>
+#include <Soprano/LiteralValue>
 
 #include <QtCore/QList>
 #include <QtCore/QHash>
@@ -34,7 +34,6 @@
 #include <QtCore/QFile>
 #include <QtCore/QUrl>
 #include <QtCore/QDebug>
-#include <QtCore/QMutex>
 #include <QtCore/QThread>
 #include <QtCore/QDateTime>
 
@@ -59,6 +58,48 @@ uint qHash( const std::string& s )
 }
 
 namespace {
+    QString findArchivePath( const QString& path ) {
+        QString p( path );
+        int i = 0;
+        while ( ( i = p.lastIndexOf( '/' ) ) > 0 ) {
+            p.truncate( i );
+            if ( QFileInfo( p ).isFile() ) {
+                return p;
+            }
+        }
+        return QString();
+    }
+
+    QUrl createResourceUri( const Strigi::AnalysisResult* idx ) {
+        // HACK: Strigi includes analysers that recurse into tar or zip archives and index
+        // the files therein. In KDE these files could perfectly be handled through kio slaves
+        // such as tar:/ or zip:/
+        // Here we try to use KDE-compatible URIs for these indexed files the best we can
+        // everything else defaults to file:/
+        QString path = QFile::decodeName( idx->path().c_str() );
+        QUrl url = QUrl::fromLocalFile( QFileInfo( path ).absoluteFilePath() );
+        if ( idx->depth() > 0 ) {
+            QString archivePath = findArchivePath( path );
+            if ( QFile::exists( archivePath ) ) {
+                if ( archivePath.endsWith( QLatin1String( ".tar" ) ) ||
+                     archivePath.endsWith( QLatin1String( ".tar.gz" ) ) ||
+                     archivePath.endsWith( QLatin1String( ".tar.bz2" ) ) ) {
+                    url.setScheme( "tar" );
+                }
+                else if ( archivePath.endsWith( QLatin1String( ".zip" ) ) ) {
+                    url.setScheme( "zip" );
+                }
+            }
+        }
+
+        // fallback for all
+        if ( url.scheme().isEmpty() ) {
+            url.setScheme( "file" );
+        }
+
+        return url;
+    }
+
     class FileMetaData
     {
     public:
@@ -67,6 +108,20 @@ namespace {
         QUrl context;
         std::string content;
     };
+
+    class RegisteredFieldData
+    {
+    public:
+        RegisteredFieldData( const QUrl& prop, QVariant::Type t )
+            : property( prop ),
+              dataType( t ),
+              isRdfType( prop == Vocabulary::RDF::type() ) {
+        }
+
+        QUrl property;
+        QVariant::Type dataType;
+        bool isRdfType;
+    };
 }
 
 
@@ -82,23 +137,22 @@ public:
         literalTypes[FieldRegister::datetimeType] = QVariant::DateTime; // Strigi encodes datetime as unsigned integer, i.e. addValue( ..., uint )
     }
 
-    QVariant::Type literalType( const std::string& strigiType ) {
-        QHash<std::string, QVariant::Type>::const_iterator it = literalTypes.find( strigiType );
+    QVariant::Type literalType( const Strigi::FieldProperties& strigiType ) {
+        // it looks as if the typeUri can contain arbitrary values, URIs or stuff like "string"
+        QHash<std::string, QVariant::Type>::const_iterator it = literalTypes.find( strigiType.typeUri() );
         if ( it == literalTypes.constEnd() ) {
-//            qDebug() << "Unknown field type: " << strigiType.c_str() << "falling back to string";
-            return QVariant::String;
+            return LiteralValue::typeFromDataTypeUri( QUrl::fromEncoded( strigiType.typeUri().c_str() ) );
         }
         else {
             return *it;
         }
     }
 
-    LiteralValue createLiteraValue( const std::string& strigiDataType,
-                                    const unsigned char* data,
-                                    uint32_t size ) {
+    LiteralValue createLiteralValue( QVariant::Type type,
+                                     const unsigned char* data,
+                                     uint32_t size ) {
         QString value = QString::fromUtf8( ( const char* )data, size );
-        QVariant::Type type = literalType( strigiDataType );
-        if ( type == QVariant::DateTime ) {
+        if ( type == QVariant::DateTime ) { // dataTime is stored as integer in strigi!
             return LiteralValue( QDateTime::fromTime_t( value.toUInt() ) );
         }
         else {
@@ -110,8 +164,6 @@ public:
     ::Soprano::Model* repository;
     int indexTransactionID;
 
-    QMutex mutex;
-
 private:
     QHash<std::string, QVariant::Type> literalTypes;
 };
@@ -129,23 +181,12 @@ Strigi::Soprano::IndexWriter::IndexWriter( ::Soprano::Model* model )
 
 Strigi::Soprano::IndexWriter::~IndexWriter()
 {
-    // just to be sure
-    commit();
-//    qDebug() << "IndexWriter::~IndexWriter in thread" << QThread::currentThread();
     delete d;
-//    qDebug() << "IndexWriter::~IndexWriter done in thread" << QThread::currentThread();
 }
 
 
 void Strigi::Soprano::IndexWriter::commit()
 {
-//    qDebug() << "IndexWriter::commit in thread" << QThread::currentThread();
-    d->mutex.lock();
-//     if ( d->indexTransactionID ) {
-//         d->repository->index()->closeTransaction( d->indexTransactionID );
-//         d->indexTransactionID = 0;
-//     }
-    d->mutex.unlock();
 }
 
 
@@ -157,7 +198,6 @@ void Strigi::Soprano::IndexWriter::deleteEntries( const std::vector<std::string>
     QString systemLocationUri = Util::fieldUri( FieldRegister::pathFieldName ).toString();
     for ( unsigned int i = 0; i < entries.size(); ++i ) {
         QString path = QString::fromUtf8( entries[i].c_str() );
-//        QString path = QString::fromUtf8( entries[i].c_str() );
         QString query = QString( "select ?g where { ?r <%1> \"%2\"^^<%3> . "
                                  "?g <http://www.strigi.org/fields#indexGraphFor> ?r . }" )
                         .arg( systemLocationUri )
@@ -208,125 +248,167 @@ void Strigi::Soprano::IndexWriter::deleteAllEntries()
 }
 
 
-// cache the field type mapping in the RegisteredFields
-void Strigi::Soprano::IndexWriter::initWriterData( const Strigi::FieldRegister& )
-{
-    // nothing to do ATM
-}
-
-
-// cleanup field type caching
-void Strigi::Soprano::IndexWriter::releaseWriterData( const Strigi::FieldRegister& )
-{
-    // nothing to do ATM
-}
-
-
 // called for each indexed file
 void Strigi::Soprano::IndexWriter::startAnalysis( const AnalysisResult* idx )
 {
+    if ( idx->depth() > 0 ) {
+        return;
+    }
+
 //    qDebug() << "IndexWriter::startAnalysis in thread" << QThread::currentThread();
     FileMetaData* data = new FileMetaData();
-    data->fileUri = Util::fileUrl( idx->path() );
-    data->context = Util::uniqueUri( "http://www.strigi.org/contexts/", d->repository );
+    data->fileUri = createResourceUri( idx );
+
+    // let's check if we already have data on the file
+    StatementIterator it = d->repository->listStatements( Node(),
+                                                          QUrl::fromEncoded( "http://www.strigi.org/fields#indexGraphFor", QUrl::StrictMode ), // FIXME: put the URI somewhere else
+                                                          data->fileUri );
+    if ( it.next() ) {
+        data->context = it.current().subject().uri();
+    }
+    else {
+        data->context = Util::uniqueUri( "http://www.strigi.org/contexts/", d->repository );
+    }
 
     qDebug() << "Starting analysis for" << data->fileUri << "in thread" << QThread::currentThread();
 
     idx->setWriterData( data );
-
-    d->mutex.lock();
-//     if ( d->indexTransactionID == 0 ) {
-//         d->indexTransactionID = d->repository->index()->startTransaction();
-//     }
-    d->mutex.unlock();
-//    qDebug() << "IndexWriter::startAnalysis done in thread" << QThread::currentThread();
 }
 
 
-// plain text accociated with the indexed file but no field name.
 void Strigi::Soprano::IndexWriter::addText( const AnalysisResult* idx, const char* text, int32_t length )
 {
+    if ( idx->depth() > 0 ) {
+        return;
+    }
+
     FileMetaData* md = reinterpret_cast<FileMetaData*>( idx->writerData() );
     md->content.append( text, length );
 }
 
 
-// convenience method for adding string fields
 void Strigi::Soprano::IndexWriter::addValue( const AnalysisResult* idx,
-                                             const RegisteredField* fieldname,
+                                             const RegisteredField* field,
                                              const std::string& value )
 {
-    addValue( idx, fieldname, ( unsigned char* )value.c_str(), value.length() );
-}
-
-
-// the main addValue method
-void Strigi::Soprano::IndexWriter::addValue( const AnalysisResult* idx,
-                                             const RegisteredField* fieldname,
-                                             const unsigned char* data,
-                                             uint32_t size )
-{
-//    qDebug() << "IndexWriter::addValue in thread" << QThread::currentThread();
-    FileMetaData* md = reinterpret_cast<FileMetaData*>( idx->writerData() );
-
-    if ( d->literalType( fieldname->type() ) == QVariant::Invalid ) {
-        // should not happen
+    if ( idx->depth() > 0 ) {
+        return;
     }
-    else {
-        d->repository->addStatement( Statement( md->fileUri,
-                                                Util::fieldUri( fieldname->key() ),
-                                                d->createLiteraValue( fieldname->type(), data, size ),
-                                                md->context) );
+
+//    qDebug() << "IndexWriter::addValue in thread" << QThread::currentThread();
+    if ( value.length() > 0 ) {
+        FileMetaData* md = reinterpret_cast<FileMetaData*>( idx->writerData() );
+        RegisteredFieldData* rfd = reinterpret_cast<RegisteredFieldData*>( field->writerData() );
+
+        if ( rfd->isRdfType ) {
+
+            // Strigi uses rdf:type improperly since it stores the value as a string. We have to
+            // make sure it is a resource. The problem is that this results in the type not being
+            // indexed properly. Thus, it cannot be searched with normal lucene queries.
+            // That is why we need to introduce a stringType property
+
+            d->repository->addStatement( Statement( md->fileUri,
+                                                    ::Soprano::Vocabulary::RDF::type(),
+                                                    QUrl::fromEncoded( value.c_str(), QUrl::StrictMode ), // fromEncoded is faster than the plain constructor and all Xesam URIs work here
+                                                    md->context) );
+            d->repository->addStatement( Statement( md->fileUri,
+                                                    QUrl::fromEncoded( "http://strigi.sourceforge.net/fields#rdf-string-type", QUrl::StrictMode ),
+                                                    LiteralValue( QString::fromUtf8( value.c_str() ) ),
+                                                    md->context) );
+        }
+
+        else {
+            if ( rfd->dataType != QVariant::Invalid ) {
+                d->repository->addStatement( Statement( md->fileUri,
+                                                        rfd->property,
+                                                        d->createLiteralValue( rfd->dataType, ( unsigned char* )value.c_str(), value.length() ),
+                                                        md->context) );
+            }
+            else {
+                qDebug() << "Ignoring field" << rfd->property << "due to unknown type" << field->properties().typeUri().c_str();
+            }
+        }
     }
 //    qDebug() << "IndexWriter::addValue done in thread" << QThread::currentThread();
 }
 
 
-// convenience method for adding unsigned int (or datetime!) fields
+// the main addValue method
 void Strigi::Soprano::IndexWriter::addValue( const AnalysisResult* idx,
-                                             const RegisteredField* fieldname,
+                                             const RegisteredField* field,
+                                             const unsigned char* data,
+                                             uint32_t size )
+{
+    addValue( idx, field, std::string( ( const char* )data, size ) );
+}
+
+
+void Strigi::Soprano::IndexWriter::addValue( const AnalysisResult*, const RegisteredField*,
+                                             const std::string&, const std::string& )
+{
+    // we do not support map types
+}
+
+
+void Strigi::Soprano::IndexWriter::addValue( const AnalysisResult* idx,
+                                             const RegisteredField* field,
                                              uint32_t value )
 {
+    if ( idx->depth() > 0 ) {
+        return;
+    }
+
 //    qDebug() << "IndexWriter::addValue in thread" << QThread::currentThread();
     FileMetaData* md = reinterpret_cast<FileMetaData*>( idx->writerData() );
+    RegisteredFieldData* rfd = reinterpret_cast<RegisteredFieldData*>( field->writerData() );
+
     LiteralValue val( value );
-    if ( fieldname->type() == FieldRegister::datetimeType ) {
-//        qDebug() << "(Soprano::IndexWriter) adding datetime value.";
+    if ( field->type() == FieldRegister::datetimeType ) {
         val = QDateTime::fromTime_t( value );
     }
 
     d->repository->addStatement( Statement( md->fileUri,
-                                            Util::fieldUri( fieldname->key() ),
+                                            rfd->property,
                                             val,
                                             md->context) );
 //    qDebug() << "IndexWriter::addValue done in thread" << QThread::currentThread();
 }
 
 
-// convenience method for adding int fields
 void Strigi::Soprano::IndexWriter::addValue( const AnalysisResult* idx,
-                                             const RegisteredField* fieldname,
+                                             const RegisteredField* field,
                                              int32_t value )
 {
+    if ( idx->depth() > 0 ) {
+        return;
+    }
+
 //    qDebug() << "IndexWriter::addValue in thread" << QThread::currentThread();
     FileMetaData* md = reinterpret_cast<FileMetaData*>( idx->writerData() );
+    RegisteredFieldData* rfd = reinterpret_cast<RegisteredFieldData*>( field->writerData() );
+
     d->repository->addStatement( Statement( md->fileUri,
-                                            Util::fieldUri( fieldname->key() ),
+                                            rfd->property,
                                             LiteralValue( value ),
                                             md->context) );
 //    qDebug() << "IndexWriter::addValue done in thread" << QThread::currentThread();
 }
 
 
-// convenience method for adding double fields
 void Strigi::Soprano::IndexWriter::addValue( const AnalysisResult* idx,
-                                             const RegisteredField* fieldname,
+                                             const RegisteredField* field,
                                              double value )
 {
+    if ( idx->depth() > 0 ) {
+        return;
+    }
+
 //    qDebug() << "IndexWriter::addValue in thread" << QThread::currentThread();
     FileMetaData* md = reinterpret_cast<FileMetaData*>( idx->writerData() );
+    RegisteredFieldData* rfd = reinterpret_cast<RegisteredFieldData*>( field->writerData() );
+
     d->repository->addStatement( Statement( md->fileUri,
-                                            Util::fieldUri( fieldname->key() ),
+                                            rfd->property,
                                             LiteralValue( value ),
                                             md->context) );
 //    qDebug() << "IndexWriter::addValue done in thread" << QThread::currentThread();
@@ -336,7 +418,7 @@ void Strigi::Soprano::IndexWriter::addValue( const AnalysisResult* idx,
 void Strigi::Soprano::IndexWriter::addTriplet( const std::string& subject,
                                                const std::string& predicate, const std::string& object )
 {
-    // PROBLEM: which names graph (context) should we use here? Create a new one for each triple? Use one until the
+    // PROBLEM: which named graph (context) should we use here? Create a new one for each triple? Use one until the
     // next commit()?
 
     // FIXME: create an NRL metadata graph
@@ -347,40 +429,31 @@ void Strigi::Soprano::IndexWriter::addTriplet( const std::string& subject,
 }
 
 
-void Strigi::Soprano::IndexWriter::addValue( const AnalysisResult* idx, const RegisteredField* field,
-                                             const std::string& name, const std::string& value )
-{
-//    qDebug() << "IndexWriter::addValue in thread" << QThread::currentThread();
-    FileMetaData* md = reinterpret_cast<FileMetaData*>( idx->writerData() );
-
-    if ( d->literalType( field->type() ) == QVariant::Invalid ) {
-        // FIXME: only save it in the index: binary data (how does strigi handle that anyway??)
-    }
-    else {
-        d->repository->addStatement( Statement( md->fileUri,
-                                                Util::fieldUri( name ),
-                                                d->createLiteraValue( field->type(), ( unsigned char* )value.c_str(), value.length() ),
-                                                md->context) );
-    }
-//    qDebug() << "IndexWriter::addValue done in thread" << QThread::currentThread();
-}
-
-
 // called after each indexed file
 void Strigi::Soprano::IndexWriter::finishAnalysis( const AnalysisResult* idx )
 {
+    if ( idx->depth() > 0 ) {
+        return;
+    }
+
 //    qDebug() << "IndexWriter::finishAnalysis in thread" << QThread::currentThread();
     FileMetaData* md = static_cast<FileMetaData*>( idx->writerData() );
 
-    // FIXME: should we store the resource type? for example nie:File or Xesam:File or whatever?
-    // FIXME: Does Xesam split file and content resources? Should we do that here, too?
+    if ( md->content.length() > 0 ) {
+        d->repository->addStatement( Statement( md->fileUri,
+                                                Vocabulary::Xesam::asText(),
+                                                LiteralValue( QString::fromUtf8( md->content.c_str() ) ),
+                                                md->context ) );
+    }
 
-    // FIXME: index the content data. Storing it would take way too much space.
+    // Strigi only indexes files and extractors mostly (if at all) store the xesam:DataObject type (i.e. the contents)
+    // Thus, here we go the easy way and mark each indexed file as a xesam:File.
     d->repository->addStatement( Statement( md->fileUri,
-                                            Util::fieldUri( FieldRegister::contentFieldName ),
-                                            LiteralValue( QString::fromUtf8( md->content.c_str() ) ),
+                                            Vocabulary::RDF::type(),
+                                            Vocabulary::Xesam::File(),
                                             md->context ) );
 
+
     // create the provedance data for the data graph
     // TODO: add more data at some point when it becomes of interest
     QUrl metaDataContext = Util::uniqueUri( "http://www.strigi.org/graphMetaData/", d->repository );
@@ -393,7 +466,7 @@ void Strigi::Soprano::IndexWriter::finishAnalysis( const AnalysisResult* idx )
                                             LiteralValue( QDateTime::currentDateTime() ),
                                             metaDataContext ) );
     d->repository->addStatement( Statement( md->context,
-                                            QUrl( "http://www.strigi.org/fields#indexGraphFor" ), // FIXME: put the URI somewhere else
+                                            QUrl::fromEncoded( "http://www.strigi.org/fields#indexGraphFor", QUrl::StrictMode ), // FIXME: put the URI somewhere else
                                             md->fileUri,
                                             metaDataContext ) );
     d->repository->addStatement( Statement( metaDataContext,
@@ -407,3 +480,28 @@ void Strigi::Soprano::IndexWriter::finishAnalysis( const AnalysisResult* idx )
 
 //    qDebug() << "IndexWriter::finishAnalysis done in thread" << QThread::currentThread();
 }
+
+
+void Strigi::Soprano::IndexWriter::initWriterData( const Strigi::FieldRegister& f )
+{
+    map<string, RegisteredField*>::const_iterator i;
+    map<string, RegisteredField*>::const_iterator end = f.fields().end();
+    for (i = f.fields().begin(); i != end; ++i) {
+        QUrl prop = Util::fieldUri( i->second->key() );
+        i->second->setWriterData( new RegisteredFieldData( prop,
+                                                           prop == Vocabulary::RDF::type()
+                                                           ? QVariant::Invalid
+                                                           : d->literalType( i->second->properties() ) ) );
+    }
+}
+
+
+void Strigi::Soprano::IndexWriter::releaseWriterData( const Strigi::FieldRegister& f )
+{
+    map<string, RegisteredField*>::const_iterator i;
+    map<string, RegisteredField*>::const_iterator end = f.fields().end();
+    for (i = f.fields().begin(); i != end; ++i) {
+        delete static_cast<RegisteredFieldData*>( i->second->writerData() );
+        i->second->setWriterData( 0 );
+    }
+}
